#include<IRremote.h>
#include "HCSR04.h"
#include <Servo.h>

// Testing Variables
int turn_distance = 15;         // cm
int servo_angle_addition = 5;  // adds or subtract from servo angle

int delay1 = 1000;                // Reverse delay
int delay2 = 0;                 // Turn delay


// IR Remote
#define IR_Pin 2            // IR receiver Pin
IRrecv irrecv(IR_Pin);      //instantiate a IR receiver object
decode_results results;     //instantiate a decode_results object. This object is separate from the IR receiver.

// IR Varaibles
int ch_minus = 16753245;    // Turns on/off the roomba
int ch = 16736925;          // Start/stop the Fan
int ch_plus = 16761405;     // Start/stop the motors

int left_arrow = 16720605;  // Decrease the motor speed
int right_arrow = 16712445; // Increase the motor speed

int minus = 16769055;       // Decrease the Fan speed
int plus = 16754775;        // Increase the Fan speed


// Program Variables
int Program = 0;            // The program is off


// DC Motor Controls


#define speedPinA 3         // PWM Motor 1 Control
#define dir1PinA 4          // Motor 1 Control 1
#define dir2PinA 5          // Motor 1 Control 2

#define speedPinB 6         // PWM Motor 2 Control
#define dir1PinB 7          // Motor 2 Control 1
#define dir2PinB 8          // Motor 2 Control 2

// DC Motor Speeds
int speedMotor = 255;       // Intial Motor Speed
int speedMotor_high = 255;  // Highest Motor Speed
int speedMotor_low = 0;     // Stops the Speed


// DC Fan
#define speedFanPin 9       // PWM Fan Control
#define dir1Fan 10          // Trun on/off the Fan
int speedFan = 255;         // Intial Fan Speed
int speedFan_high = 255;    // Highest Speed for Fan
int speedFan_low = 0;       // Stops the Fan



// Loading the Sonar Sensor pins and library
#define echoPin 11          // This is the echo pin
#define triggerPin 12       // This is the trigger pin
HCSR04ProxSensor distanceSensor(echoPin, triggerPin); //here we call the constructor to instantiate a sensor named "distanceSensor"




// Loading the Servo pins and library
Servo myservo;              // create servo object to control a servo
// twelve servo objects can be created on most boards
int angle;                  // variable to store the servo position
int right_left;             // determine if the servo needs to go to the left or right
int side;                   // determine the side the servo is on
const float pi = 3.14;      // pi
float x_distance;
float radians1;





void setup() {
  Serial.begin(9600);       //start serial communication
  // IR initial Code
  irrecv.enableIRIn();      // Start the receiver

  // Servo Intial Code
  myservo.attach(13);       // attaches the servo on pin 9 to the servo object
  angle = 45;                // variable to store the servo position
  right_left = 1;           // determine if the servo needs to go to the left or right
  side = 1;                 // determine the side the servo is on
  x_distance = 0;
  radians1 = 0;
  myservo.write(90);
}

void loop() {
  // Collect The IR Signal
  if (irrecv.decode(&results)) {//has a transmission been received?
    if (Program == 0 && results.value == 16753245) { //Ch-
      //Turns on all functions
      Program = 1;
          // Turn Fan on
          speedFan = speedFan_high;
          digitalWrite(dir1Fan,HIGH);
          analogWrite(speedFanPin,speedFan);

          // Turn on Motors
          speedMotor = speedMotor_high;
          // Turn off Pins 1
          digitalWrite(dir1PinA,LOW);
          digitalWrite(dir1PinB,LOW);
          // Turn on Pins 2
          digitalWrite(dir2PinA,HIGH);
          digitalWrite(dir2PinB,HIGH);
          // Speed Controls
          analogWrite(speedPinA,speedMotor);
          analogWrite(speedPinB,speedMotor);
  
    }
    if (Program == 1) {
      if (results.value == 16736925) {          //Ch
        //Turns off the program
        Program = 0;
        // Fan off
        digitalWrite(dir1Fan,LOW);
        // Motors off
        digitalWrite(dir1PinA,LOW);
        digitalWrite(dir1PinB,LOW);
        // Turn on Pins 2
        digitalWrite(dir2PinA,LOW);
        digitalWrite(dir2PinB,LOW);
      }
      // Stops the Motor
      if (results.value == 16720605) {          //<<
        speedMotor = speedMotor_low;
        analogWrite(speedPinA,speedMotor);
        analogWrite(speedPinB,speedMotor);
      }
      // Starts the Motor
      if (results.value == 16712445) {          //>>
        speedMotor = speedMotor_high;
        analogWrite(speedPinA,speedMotor);
        analogWrite(speedPinB,speedMotor);
      }
      // Stops the Fan
      if (results.value == 16769055) {          //-
        speedFan = speedFan_low;
        analogWrite(speedFanPin,speedFan);
      }
      // Starts the Fan
      if (results.value == 16754775) {          //+
        speedFan = speedFan_high;
       analogWrite(speedFanPin,speedFan);
      }
    }
    irrecv.resume(); // Receive the next value

  }
  // Servo Controls
  if (Program == 1) {
    // If the servo needs to go to the right and less then 180 degrees
    if (right_left == 1 && angle < 135) {
      // Add degrees
      angle = angle + servo_angle_addition;
      myservo.write(angle);
      // If angle equals 90 degrees then the servo switches side
      if (angle == 90) {
        side = 0;
      }
      // If the angle equals 180 sweep to the left
      if (angle == 135) {
        right_left = 0;
      }
    }

    // If the servo needs to go to the right and less then 180 degrees
    if (right_left == 0 && angle > 45) {
      // Subtract degrees
      angle = angle - servo_angle_addition;
      myservo.write(angle);
      // If angle equals 90 degrees then the servo switches side
      if (angle == 90) {
        side = 1;
      }
      // If the angle equals 180 sweep to the right
      if (angle == 45) {
        right_left = 1;
      }
    }
    


    // Sonar Control
    // Reads the distance
    float distance = distanceSensor.readSensor();     //here we call the 'readSensor' method to determine the distance
    radians1 = radians(angle);                        // Converts degree to radianss
    x_distance = abs(distance * sin(radians1));       // find the distance from the object
Serial.println(x_distance);
delay(50);

    // Turn the wheels
    if (x_distance < turn_distance   && x_distance!=0) {
      if (side == 0) {
        // Reverse the motors to back away from the object
        digitalWrite(dir1PinA,HIGH);
        digitalWrite(dir1PinB,HIGH);
        // Turn off Pins 2
        digitalWrite(dir2PinA,LOW);
        digitalWrite(dir2PinB,LOW);
        
        delay(delay1);

        // Turn to the right
        // Turn off Pins 1
        digitalWrite(dir1PinA,HIGH);
        digitalWrite(dir1PinB,LOW);
        // Turn on Pins 2
        digitalWrite(dir2PinA,LOW);
        digitalWrite(dir2PinB,HIGH);
  
        delay2 = random(1, 5) * 300;
        delay(delay2);

        // Forward
        // Turn off Pins 1
        digitalWrite(dir1PinA,LOW);
        digitalWrite(dir1PinB,LOW);
        // Turn on Pins 2
        digitalWrite(dir2PinA,HIGH);
        digitalWrite(dir2PinB,HIGH);
      }
      if (side == 1) {
        // Reverse the motors to back away from the object
        digitalWrite(dir1PinA,HIGH);
        digitalWrite(dir1PinB,HIGH);
        // Turn off Pins 2
        digitalWrite(dir2PinA,LOW);
        digitalWrite(dir2PinB,LOW);
        
        delay(delay1);

        // Turn to the left
        // Turn off Pins 1
        digitalWrite(dir1PinA,LOW);
        digitalWrite(dir1PinB,HIGH);
        // Turn on Pins 2
        digitalWrite(dir2PinA,HIGH);
        digitalWrite(dir2PinB,LOW);
  
        delay2 = random(1, 5) * 300;
        delay(delay2);
        // Forward
        // Turn off Pins 1
        digitalWrite(dir1PinA,LOW);
        digitalWrite(dir1PinB,LOW);
        // Turn on Pins 2
        digitalWrite(dir2PinA,HIGH);
        digitalWrite(dir2PinB,HIGH);
      }

    }
  }
}

